/**************************************************************************************************
* File: ACCEL_driver.c
* File Type: C - Header File
* Project Name: BrmBrrrrm
* Company: (c) mikroElektronika, 2011
* Revision History:
*     20111020 (DA):
*       - initial release;
* Description:
*     This module contains a set of functions that are used for communication with
*     3-axis digital accelerometer ADXL345.
*
* NOTES:
*     I2C2 bus is used for communication with accel.
**************************************************************************************************/
char cACCEL_test_status;


void ADXL345_Write(unsigned short address, unsigned short data1);
unsigned short ADXL345_Read(unsigned short address);
char ADXL345_Init(void);

// Read X Axis
int Accel_ReadX(void);

// Read Y Axis
int Accel_ReadY(void);

// Read Z Axis
int Accel_ReadZ(void);

// ADXL345 Register Definition
#define _POWER_CTL      0x2D
#define _DATA_FORMAT    0x31
#define _BW_RATE        0x2C
#define _DATAX0         0x32
#define _DATAX1         0x33
#define _DATAY0         0x34
#define _DATAY1         0x35
#define _DATAZ0         0x36
#define _DATAZ1         0x37
#define _FIFO_CTL       0x38
#define _SPEED          0x0F          // Buffer Speed - 3200Hz

#define _ACCEL_ERROR    0x02

/**************************************************************************************************
* Function ADXL345_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write byte of data to ADXL345
* Input: register address, data
* Output: Nothing
**************************************************************************************************/
void ADXL345_Write(unsigned short address, unsigned short data1)
{
  I2C2_Start();                      // issue I2C start signal
  I2C2_Write(0x3A);                  // send byte via I2C  (device address + W)
  I2C2_Write(address);               // send byte (address of the location)
  I2C2_Write(data1);                 // send data (data to be written)
  I2C2_Stop();                       // issue I2C stop signal
}

/**************************************************************************************************
* Function ADXL345_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function read byte of data from ADXL345
* Input: register address
* Output: data from addressed register in ADXL345
**************************************************************************************************/
unsigned short ADXL345_Read(unsigned short address)
{
  unsigned short tmp = 0;

  I2C2_Start();                      // issue I2C start signal
  I2C2_Write(0x3A);                  // send byte via I2C (device address + W)
  I2C2_Write(address);               // send byte (data address)
  
  I2C2_Start();                      // issue I2C signal repeated start
  I2C2_Write(0x3B);                  // send byte (device address + R)
  tmp = I2C2_Read(1);                // Read the data (NO acknowledge)
  I2C2_Stop();                       // issue I2C stop signal

  return tmp;
}

/**************************************************************************************************
* Function ADXL345_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function that initializes ADXL345
* Input: Nothing
* Output: return 0 for OK; return _ACCEL_ERROR for ERROR
**************************************************************************************************/
char ADXL345_Init(void)
{
  char id = 0x00;

  // Go into standby mode to configure the device.
  ADXL345_Write(0x2D, 0x00);

  id = ADXL345_Read(0x00);

  if (id != 0xE5)
  {
    return _ACCEL_ERROR;
  }
  else
  {
    ADXL345_Write(_DATA_FORMAT, 0x08);       // Full resolution, +/-2g, 4mg/LSB, right justified
    ADXL345_Write(_BW_RATE, 0x0A);           // Set 100 Hz data rate
    ADXL345_Write(_FIFO_CTL, 0x80);          // stream mode
    ADXL345_Write(_POWER_CTL, 0x08);         // POWER_CTL reg: measurement mode
    return 0x00;
  }
}

/**************************************************************************************************
* Function Accel_ReadX()
* -------------------------------------------------------------------------------------------------
* Overview: Function read X axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
int Accel_ReadX(void)
{
  char low_byte;
  int Out_x;

  Out_x = ADXL345_Read(_DATAX1);
  low_byte = ADXL345_Read(_DATAX0);

  Out_x = (Out_x << 8);
  Out_x = (Out_x | low_byte);

  return Out_x;
}

/**************************************************************************************************
* Function Accel_ReadY()
* -------------------------------------------------------------------------------------------------
* Overview: Function read Y axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
int Accel_ReadY(void)
{
  char low_byte;
  int Out_y;

  Out_y = ADXL345_Read(_DATAY1);
  low_byte = ADXL345_Read(_DATAY0);

  Out_y = (Out_y << 8);
  Out_y = (Out_y | low_byte);

  return Out_y;
}

/**************************************************************************************************
* Function Accel_ReadZ()
* -------------------------------------------------------------------------------------------------
* Overview: Function read Z axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
int Accel_ReadZ(void)
{
  char low_byte;
  int Out_z;

  Out_z = ADXL345_Read(_DATAZ1);
  low_byte = ADXL345_Read(_DATAZ0);

  Out_z = (Out_z << 8);
  Out_z = (Out_z | low_byte);

  return Out_z;
}
/* Function Accel_Average()
* -------------------------------------------------------------------------------------------------
* Overview: Function calculate the average values of the X, Y and Z axis reads
* Input: Nothing
* Output: X,Y and Z values are stored in readings[] buffer
*************************************************************************************************/
int Accel_Average(void)
{
  int i, sx, sy, sz;
  double moyenne, cMoy[3],moy,eType, acc_valeur[3];
  // sum
  sx = sy = sz = 0;

  // average accelerometer reading over last 16 samples
  for (i=0; i<16; i++)
  {
    sx += Accel_ReadX();
    sy += Accel_ReadY();
    sz += Accel_ReadZ();
  }
  // average
  acc_valeur[0] = sx >> 4;
  acc_valeur[1] = sy >> 4;
  acc_valeur[2] = sz >> 4;
  moyenne = ((double)acc_valeur[0] + (double)acc_valeur[1] + (double)acc_valeur[2]) / 3;
  cMoy[0] = pow(((double)acc_valeur[0] - moyenne),2);
  cMoy[1] = pow(((double)acc_valeur[1] - moyenne),2);
  cMoy[2] = pow(((double)acc_valeur[2] - moyenne),2);
  moy = (cMoy[0] + cMoy[1] + cMoy[2]) / 3;
  eType = sqrt(moy);
  return (int)eType;
}
/**************************************************************************************************
* Function ACCEL_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize I2C bus and accel module
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void ACCEL_Start(void)
{

  TFT_Write_Text("initialisation accelerometre...", 10, 50);
  // Initialize ADXL345 accelerometer
  if (ADXL345_Init() == 0)
  {
    TFT_Write_Text("reussie.", 190, 50);
  }
  else
  {
    TFT_Write_Text("impossible.", 190, 50);
  }
}

/**************************************************************************************************
* Function Display_X_Value()
* -------------------------------------------------------------------------------------------------
* Overview: Function display average X-axis read value on TFT
* Input: value stored in readings[0] buffer
* Output: Nothing
**************************************************************************************************/
/*void Display_X_Value(void)
{
  char out[5];
  TFT_Rectangle(160, 100, 200, 115);
  TFT_Write_Text("X: ", 140, 100);
  IntToStr(acc_valeur[0], out);
  TFT_Write_Text(out, 160, 100);
}*/

/**************************************************************************************************
* Function Display_Y_Value()
* -------------------------------------------------------------------------------------------------
* Overview: Function display average Y-axis read value on TFT
* Input: value stored in readings[1] buffer
* Output: Nothing
**************************************************************************************************/
/*void Display_Y_Value(void)
{
  char out[5];
  TFT_Rectangle(160, 130, 200, 145);
  TFT_Write_Text("Y: ", 140, 130);
  IntToStr(acc_valeur[1], out);
  TFT_Write_Text(out, 160, 130);
}*/

/**************************************************************************************************
* Function Display_Z_Value()
* -------------------------------------------------------------------------------------------------
* Overview: Function display average Z-axis read value on TFT
* Input: value stored in readings[2] buffer
* Output: Nothing
**************************************************************************************************/
/*void Display_Z_Value(void)
{
  char out[5];
  TFT_Rectangle(160, 160, 200, 175);
  TFT_Write_Text("Z: ", 140, 160);
  IntToStr(acc_valeur[2], out);
  TFT_Write_Text(out, 160, 160);
}
 void Display_E_Value(int eca)
{
  char out[5];
  TFT_Rectangle(160, 190, 200, 205);
  TFT_Write_Text("E: ", 140, 190);
  intToStr(eca, out);
  TFT_Write_Text(out, 160, 190);
}*/

/**************************************************************************************************
* Function ACCEL_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the accel test
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
/*void ACCEL_Test(void)
{
int ecartype;
  TFT_Write_Text("Reading axis values :", 90, 70);
  ecartype = Accel_Average();               // Calculate average X, Y and Z reads
//  Display_X_Value();             // Display average X value read
//  Display_Y_Value();             // Display average Y value read
//  Display_Z_Value();             // Display average Z value read
  Display_E_Value(ecartype);
}*/
/**************************************************************************************************
* End of File
**************************************************************************************************/